from typing import List, Tuple

from oelint_parser.cls_item import Function, Item, PythonBlock, Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarPythonPnUsage(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.pythonpnusage',
                         severity='info',
                         message='python3 should be used instead of ${PYTHON_PN}',
                         valid_from_release='scarthgap')

    def __getMatches(self, _file: str, stash: Stash) -> List[Tuple[Item, List[str]]]:
        res = []
        items: List[Item] = stash.GetItemsFor(filename=_file,
                                              classifier=[
                                                  Variable.CLASSIFIER,
                                                  Function.CLASSIFIER,
                                                  PythonBlock.CLASSIFIER,
                                              ])
        for i in items:
            if i.Origin.endswith('.bbclass'):
                continue
            if isinstance(i, PythonBlock):
                needles = ['${PYTHON_PN}', 'd.getVar("PYTHON_PN")', 'd.getVar(\'PYTHON_PN\')']
            elif isinstance(i, (Function, Variable)):
                needles = ['${PYTHON_PN}']
            else:
                needles = []  # pragma: no cover
            if any(x in i.Raw for x in needles):
                res.append((i, needles))
        return res

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        for i in self.__getMatches(_file, stash):
            item, _ = i
            res += self.finding(item.Origin, item.InFileLine)
        return res

    def fix(self, _file: str, stash: Stash) -> List[str]:
        res = []
        for i in self.__getMatches(_file, stash):
            item, needles = i
            for needle in needles:
                item.RealRaw = item.RealRaw.replace(needle, 'python3')
                item.Raw = item.Raw.replace(needle, 'python3')
            res.append(_file)
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.rpl_regex import RegexRpl

from oelint_adv.cls_rule import Rule


class VarSectionLowercase(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.notneededspace',
                         severity='info',
                         message='Space at the beginning of the var is not needed')

    def __getMatches(self, _file: str, stash: Stash) -> List[Variable]:
        res = []
        items = stash.GetItemsFor(
            filename=_file, classifier=Variable.CLASSIFIER)
        for i in [x for x in items if 'append' not in x.SubItems]:
            if i.VarValueStripped.startswith(' '):
                res.append(i)
        return res

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        for i in self.__getMatches(_file, stash):
            res += self.finding(i.Origin, i.InFileLine)
        return res

    def fix(self, _file: str, stash: Stash) -> List[str]:
        res = []
        for i in self.__getMatches(_file, stash):
            i.RealRaw = RegexRpl.sub(r'"\s+', '"', i.RealRaw) + '\n'
            i.Raw = RegexRpl.sub(r'"\s+', '"', i.Raw) + '\n'
            i.VarValue = RegexRpl.sub(r'"\s+', '"', i.VarValue) + '\n'
            res.append(_file)
        return res



from .base import TestBaseClass

import os


# flake8: noqa S101 - n.a. for test files
class TestGrouping(TestBaseClass):

    def __flatten_file_names(self, list_):
        return [os.path.basename(x) for x in list_]

    def test_fast_bb_1(self):
        from oelint_adv.core import group_files

        input_ = {
            'test1.bb': '',
            'test2.bb': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['test1.bb'] in res
        assert ['test2.bb'] in res

    def test_fast_bb_bbappend(self):
        from oelint_adv.core import group_files

        input_ = {
            'test1.bb': '',
            'test1.bbappend': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['test1.bb', 'test1.bbappend'] in res

    def test_fast_bb_bbappend_version(self):
        from oelint_adv.core import group_files

        input_ = {
            'test1_1.0.bb': '',
            'test1_%.bbappend': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['test1_1.0.bb', 'test1_%.bbappend'] in res

    def test_fast_bb_bbappend_version_2(self):
        from oelint_adv.core import group_files

        input_ = {
            'test1_1.0.bb': '',
            'test1_1.%.bbappend': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['test1_1.0.bb', 'test1_1.%.bbappend'] in res

    def test_fast_bb_bbappend_lone(self):
        from oelint_adv.core import group_files

        input_ = {
            'test1_1.0.bb': '',
            'test1_2.%.bbappend': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['test1_1.0.bb'] in res
        assert ['test1_2.%.bbappend'] in res

    def test_fast_layer_conf(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/layer.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['layer.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf'] in res

    def test_all_layer_conf(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/layer.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': ''
        }

        args = self._create_args(input_, ['--mode=all'])
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['layer.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf'] in res

    def test_fast_distro_conf(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/distro/mydistro.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['test1_1.0.bb'] in res
        assert ['test2_1.0.bb'] in res
        assert ['mydistro.conf'] not in res

    def test_all_distro_conf(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/distro/mydistro.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': ''
        }

        args = self._create_args(input_, ['--mode=all'])
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['mydistro.conf', 'test1_1.0.bb'] in res
        assert ['mydistro.conf', 'test2_1.0.bb'] in res
        assert ['mydistro.conf'] in res

    def test_fast_machine_conf(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/machine/mymachine.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': ''
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['test1_1.0.bb'] in res
        assert ['test2_1.0.bb'] in res
        assert ['mymachine.conf'] not in res

    def test_all_machine_conf(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/machine/mymachine.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': ''
        }

        args = self._create_args(input_, ['--mode=all'])
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['mymachine.conf', 'test1_1.0.bb'] in res
        assert ['mymachine.conf', 'test2_1.0.bb'] in res
        assert ['mymachine.conf'] in res

    def test_fast_full_setup(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/distro/mydistro.conf': '',
            'conf/machine/mymachine.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': '',
            'conf/layer.conf': '',
        }

        args = self._create_args(input_)
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['layer.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf'] in res
        assert ['mymachine.conf'] not in res
        assert ['mydistro.conf'] not in res

    def test_all_full_setup(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/distro/mydistro.conf': '',
            'conf/machine/mymachine.conf': '',
            'test1_1.0.bb': '',
            'test2_1.0.bb': '',
            'conf/layer.conf': '',
        }

        args = self._create_args(input_, ['--mode=all'])
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['layer.conf', 'mymachine.conf', 'mydistro.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'mymachine.conf', 'mydistro.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf'] in res
        assert ['mymachine.conf'] in res
        assert ['mydistro.conf'] in res

    def test_all_full_setup_multiple(self):
        from oelint_adv.core import group_files

        input_ = {
            'conf/distro/mydistro.conf': '',
            'conf/machine/mymachine.conf': '',
            'test1_1.0.bb': '',
            'conf/distro/mydistro2.conf': '',
            'conf/machine/mymachine2.conf': '',
            'test2_1.0.bb': '',
            'conf/layer.conf': '',
        }

        args = self._create_args(input_, ['--mode=all'])
        res = [self.__flatten_file_names(x[0]) for x in group_files(args.files, args.mode)]

        assert ['layer.conf', 'mymachine.conf', 'mydistro.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'mymachine2.conf', 'mydistro.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'mymachine.conf', 'mydistro2.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'mymachine2.conf', 'mydistro2.conf', 'test1_1.0.bb'] in res
        assert ['layer.conf', 'mymachine.conf', 'mydistro.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf', 'mymachine2.conf', 'mydistro.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf', 'mymachine.conf', 'mydistro2.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf', 'mymachine2.conf', 'mydistro2.conf', 'test2_1.0.bb'] in res
        assert ['layer.conf'] in res
        assert ['mymachine.conf'] in res
        assert ['mymachine2.conf'] in res
        assert ['mydistro.conf'] in res
        assert ['mydistro2.conf'] in res



import pytest  # noqa: I900
import os
import glob

from .base import TestBaseClass


# flake8: noqa S101 - n.a. for test files
class TestCached(TestBaseClass):

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'test.bb':
                                     '''
                                     INSANE_SKIP:${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_cached(self, capsys, input_):

        from oelint_adv.core import run
        from oelint_adv.__main__ import arguments_post

        tmpdir = os.path.dirname(self._create_tempfile('tmppath/.marker', ''))
        files = [self._create_tempfile(k, v) for k, v in input_.items()]

        args = arguments_post(self._create_args_parser().parse_args(
            ['--jobs=1', '--cached', f'--cachedir={tmpdir}', *files]
        ))

        run(args)

        captured = capsys.readouterr()

        assert 'Using cached item ' not in captured.out

        assert any(glob.glob(f'{tmpdir}/*'))

        run(args)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'test.bb':
                                     '''
                                     INSANE_SKIP:${PN} = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_no_cached(self, capsys, input_):

        from oelint_adv.core import run
        from oelint_adv.__main__ import arguments_post

        tmpdir = os.path.dirname(self._create_tempfile('tmppath/.marker', ''))
        files = [self._create_tempfile(k, v) for k, v in input_.items()]

        args = arguments_post(self._create_args_parser().parse_args(
            ['--jobs=1', *files]
        ))

        run(args)

        captured = capsys.readouterr()

        assert 'Using cached item ' not in captured.out

        assert not any(glob.glob(f'{tmpdir}/*'))

    def test_clear_cached(self):

        from oelint_adv.__main__ import arguments_post

        tmpdir = os.path.dirname(self._create_tempfile('tmppath/.marker', ''))

        args = arguments_post(self._create_args_parser().parse_args(
            ['--jobs=1', '--cached', f'--cachedir={tmpdir}', '--clear-caches']
        ))

        args.state._caches.ClearCaches()

    def test_arg_fingerprint(self):

        from oelint_adv.__main__ import arguments_post

        tmpdir = os.path.dirname(self._create_tempfile('tmppath/.marker', ''))

        args = arguments_post(self._create_args_parser().parse_args(
            ['--jobs=1', '--cached', f'--cachedir={tmpdir}', self._create_tempfile('test.bb', '')]
        ))

        fp = args.state._caches.FingerPrint

        args.state._caches.AddToFingerPrint(b'1234')

        assert args.state._caches.FingerPrint != fp

        args.state._caches.AddToFingerPrint('1234')

        assert args.state._caches.FingerPrint != fp



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarSuggestedVar(TestBaseClass):

    def __generate_sample_code(self, var, extra):
        return '''
            {extra}
            {var} = "foo"
            '''.format(var=var, extra=extra)

    @pytest.mark.parametrize('id_', ['oelint.var.suggestedvar'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'BUGTRACKER',
        'CVE_PRODUCT',
        'SECTION',
    ])
    def test_bad(self, id_, occurrence, var):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code('A', ''),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.suggestedvar.CVE_PRODUCT'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint-adv_test.bb':
                                     'VAR = "a"',
                                 },
                             ],
                             )
    def test_suppress(self, id_, occurrence, input_):
        _x = self._create_args(input_, extraopts=['--suppress', id_])
        self.check_for_id(_x, id_, occurrence)
        self.check_for_id(_x, 'oelint.var.suggestedvar.BUGTRACKER', 1)

    @pytest.mark.parametrize('id_', [
        'oelint.var.suggestedvar.BUGTRACKER',
        'oelint.var.suggestedvar.BBCLASSEXTEND',
        'oelint.var.suggestedvar.CVE_PRODUCT',
        'oelint.var.suggestedvar.SECTION',
    ])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     BUGTRACKER = "1"
                                     BBCLASSEXTEND = "1"
                                     CVE_PRODUCT = "1"
                                     SECTION = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', [
        'oelint.var.suggestedvar.BUGTRACKER',
    ])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     CVE_PRODUCT = "my/my"
                                     SECTION = "foo"
                                     inherit foo
                                     ''',
                                 },
                             ],
                             )
    def test_good_custom_varname_class(self, input_, id_, occurrence):
        __cnt = '''
        {
            "oelint-suggestedvar": {
                "BUGTRACKER-exclude-classes": [
                    "foo"
                ]
            }
        }
        '''
        _extra_opts = [
            '--constantmod=+{mod}'.format(mod=self._create_tempfile('constmod', __cnt))]
        self.check_for_id(self._create_args(input_, _extra_opts), id_, occurrence)

    @pytest.mark.parametrize('id_', [
        'oelint.var.suggestedvar.BUGTRACKER',
    ])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     CVE_PRODUCT = "my/my"
                                     SECTION = "foo"
                                     inherit image
                                     ''',
                                 },
                             ],
                             )
    def test_good_custom_image(self, input_, id_, occurrence):
        __cnt = '''
        {
            "oelint-suggestedvar": {
                "image-excludes": [
                    "BUGTRACKER"
                ]
            }
        }
        '''
        _extra_opts = [
            '--constantmod=+{mod}'.format(mod=self._create_tempfile('constmod', __cnt))]
        self.check_for_id(self._create_args(input_, _extra_opts), id_, occurrence)

    @pytest.mark.parametrize('id_', [
        'oelint.var.suggestedvar.LICENSE',
    ])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     CVE_PRODUCT = "my/my"
                                     SECTION = "foo"
                                     inherit packagegroup
                                     ''',
                                 },
                             ],
                             )
    def test_good_custom_packagegroup(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsDUsagePkgFunc(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.dusageinpkgfuncs'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_preinst:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_postinst:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_prerm:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_postrm:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.dusageinpkgfuncs'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_preinst:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_postinst:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_prerm:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_postrm:${PN} () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.dusageinpkgfuncs'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_preinst:${PN} () {
                                         if [ -n "$D" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_postinst:${PN} () {
                                         if [ -n "$D" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_prerm:${PN} () {
                                         if [ -n "$D" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_postrm:${PN} () {
                                         if [ -n "$D" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     pkg_postrm:${PN} () {
                                         if [ -n "${Z}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_install () {
                                         if [ -n "${D}" ]; then
                                             echo "Foo"
                                         fi
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import tempfile

from .base import TestBaseClass


# flake8: noqa S101 - n.a. for test files
class TestLibMode(TestBaseClass):

    def test_libmode(self):
        test_recipe = '''
        INSANE_SKIP:${PN} = "foo"
        '''

        tmp_file = self._create_tempfile('test.bb', test_recipe)

        from oelint_adv.core import create_lib_arguments, run

        results = run(create_lib_arguments([tmp_file]))

        assert isinstance(results, list)
        assert len(results) > 0

        issues = [x[1] for x in results]

        assert len([x for x in issues if ':{id}:'.format(id='oelint.vars.insaneskip') in x]) == 1



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarPnBpnUsage(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.pbpusage',
                         severity='error',
                         message='${BP} should be used instead of ${P}')

    def __getMatches(self, _file: str, stash: Stash) -> List[Variable]:
        res = []
        items = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                  attribute=Variable.ATTR_VAR, attributeValue=['SRC_URI', 'S'])
        for i in items:
            for x in i.get_items():
                if i.VarName == 'SRC_URI':
                    _haystack = stash.GetScrComponents(x)['src']
                else:
                    _haystack = x
                if '${P}' in _haystack:
                    res.append(i)
        return res

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        for i in self.__getMatches(_file, stash):
            res += self.finding(i.Origin, i.InFileLine)
        return res

    def fix(self, _file: str, stash: Stash) -> List[str]:
        res = []
        for i in self.__getMatches(_file, stash):
            i.RealRaw = i.RealRaw.replace('${P}', '${BP}')
            i.Raw = i.Raw.replace('${P}', '${BP}')
            res.append(_file)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsInsaneSkip(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.insaneskip'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'INSANE_SKIP:${PN} = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'INSANE_SKIP:bla = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'INSANE_SKIP:bla:class-native = "a"',
                                 },

                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.insaneskip'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'INSANE_SKIP_${PN} = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'INSANE_SKIP_bla = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'INSANE_SKIP_bla_class-native = "a"',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.insaneskip'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR += "ffjjj"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintTaskHeredocs(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.task.heredocs'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_foo() {
                                         cat    >  ${T}/some.files <<   abchhehdhhe
                                         kfkdfkd
                                         abchhehdhhe
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_install_append() {
                                         cat    <<   EOF    >${T}/some.files
                                         abc
                                         EOF
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.heredocs'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_install() {
                                         abc
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarAutorev(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.autorev',
                         severity='warning',
                         message='The usage of \'AUTOREV\' for SRCREV leads to not reproducible builds')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR)
        for i in [x for x in items if x.VarName.startswith('SRCREV')]:
            if i.VarValueStripped == '${AUTOREV}':
                res += self.finding(i.Origin, i.InFileLine)
        return res



import os
import json

from typing import List
from oelint_parser.cls_stash import Stash
datapath = os.path.dirname(__file__)


def known_variable_mod(release: str, layer: str) -> str:
    _file = os.path.join(datapath, release, f'{layer}.json')
    if os.path.exists(_file):
        return _file
    return ''


def layer_var_mods(files: str, release: str, external_layers: List[str]) -> List[str]:
    res = []
    imports = external_layers
    _specific = f'oelint.constants.{release}.json'
    _default = 'oelint.constants.json'
    _needles = {Stash(quiet=True).GetLayerRoot(x) for x in files}

    def get_imports(_file):
        with open(_file) as i:
            try:
                return json.load(i).get('$imports', [])
            except json.JSONDecodeError:  # pragma: no cover
                return []  # pragma: no cover

    for path in _needles:
        if not path:
            continue
        if os.path.exists(os.path.join(path, _specific)):
            _new_file = os.path.join(path, _specific)
            imports.extend(get_imports(_new_file))
            res.append(f'+{_new_file}')
        elif os.path.exists(os.path.join(path, _default)):
            _new_file = os.path.join(path, _default)
            imports.extend(get_imports(_new_file))
            res.append(f'+{_new_file}')
    return (res, imports)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsNoTrailingSlash(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.notrailingslash'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORDKIR}/foo/"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'D = "${WORDKIR}/foo/"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'T = "${WORDKIR}/foo/"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'B = "${WORDKIR}/foo/"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     BAR = "foo/"
                                     S = "${WORDKIR}/${BAR}"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.notrailingslash'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORDKIR}/foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'D = "${WORDKIR}/foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'T = "${WORDKIR}/foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'B = "${WORDKIR}/foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     BAR = "foo"
                                     S = "${WORDKIR}/${BAR}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SOMEOTHERVAR = "foo/"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsPythonPnUsage(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.pythonpnusage'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "${PYTHON_PN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "${PYTHON_PN}/${PYTHON_PN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_${PYTHON_PN} = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_something() {
                                        echo ${PYTHON_PN} > test.txt
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar("PYTHON_PN")
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar(\'PYTHON_PN\')
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.expandVar("${PYTHON_PN}/A")
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pythonpnusage'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "${PYTHON_PN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "${PYTHON_PN}/${PYTHON_PN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_${PYTHON_PN} = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_something() {
                                        echo ${PYTHON_PN} > test.txt
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar("PYTHON_PN")
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar(\'PYTHON_PN\')
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.expandVar("${PYTHON_PN}/A")
                                     ''',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=nanbield']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pythonpnusage'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "${PYTHON_PN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "${PYTHON_PN}/${PYTHON_PN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_${PYTHON_PN} = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_something() {
                                        echo ${PYTHON_PN} > test.txt
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar("PYTHON_PN")
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar(\'PYTHON_PN\')
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.expandVar("${PYTHON_PN}/A")
                                     ''',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_, ['--release=scarthgap']), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.pythonpnusage'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORDKIR}/${BP}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRCREV_PYTHON_PN = "01234567890abcdef"',
                                 },
                                 {
                                     'classes/test.bbclass':
                                     'A = "${PYTHON_PN}"',
                                     'oelint_adv_test.bb':
                                     'inherit test',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "python3"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "python3/python3"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_python3 = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_something() {
                                        echo python3 > test.txt
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar("somethingelse")
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.getVar(\'somethingelse\')
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     def do_something():
                                        d.expandVar("python3/A")
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsFileSettingsHidden(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.hidden'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SPLITPKGS = "${PN}-ping ${PN}-arping ${PN}-tracepath ${PN}-clockdiff \
                                                 ${PN}-tftpd ${PN}-rdisc \
                                                 ${@bb.utils.contains('PACKAGECONFIG', 'rarpd', '${PN}-rarpd', '', d)} \
                                                 ${@bb.utils.contains('DISTRO_FEATURES', 'ipv6', '${PN}-traceroute6 ${PN}-ninfod', '', d)}"
                                     PACKAGES += "${SPLITPKGS}"
                                     ALLOW_EMPTY:${PN} = "1"
                                     RDEPENDS_${PN} += "${SPLITPKGS}"
                                     FILES:${PN} += "${bindir}"
                                     FILES:${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     FILES:${PN}-arping = "${base_bindir}/arping"
                                     FILES:${PN}-tracepath = "${base_bindir}/tracepath"
                                     FILES:${PN}-traceroute6	= "${base_bindir}/traceroute6"
                                     FILES:${PN}-clockdiff = "${base_bindir}/clockdiff"
                                     FILES:${PN}-tftpd = "${bindir} ${base_bindir}/tftpd"
                                     FILES:${PN}-rarpd = "${base_sbindir}/rarpd  ${systemd_unitdir}/system/rarpd@.service"
                                     FILES:${PN}-rdisc = "${base_sbindir}/rdisc"
                                     FILES:${PN}-ninfod = "${base_sbindir}/ninfod ${sysconfdir}/init.d/ninfod.sh"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.hidden'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SPLITPKGS = "${PN}-ping ${PN}-arping ${PN}-tracepath ${PN}-clockdiff \
                                      ${PN}-tftpd ${PN}-rdisc \
                                      ${@bb.utils.contains('PACKAGECONFIG', 'rarpd', '${PN}-rarpd', '', d)} \
                                      ${@bb.utils.contains('DISTRO_FEATURES', 'ipv6', '${PN}-traceroute6 ${PN}-ninfod', '', d)}"
                                     PACKAGES =+ "${SPLITPKGS}"

                                     ALLOW_EMPTY_${PN} = "1"
                                     RDEPENDS_${PN} += "${SPLITPKGS}"

                                     FILES_${PN} += "${bindir}"
                                     FILES_${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     FILES_${PN}-arping = "${base_bindir}/arping"
                                     FILES_${PN}-tracepath = "${base_bindir}/tracepath"
                                     FILES_${PN}-traceroute6	= "${base_bindir}/traceroute6"
                                     FILES_${PN}-clockdiff = "${base_bindir}/clockdiff"
                                     FILES_${PN}-tftpd = "${bindir} ${base_bindir}/tftpd"
                                     FILES_${PN}-rarpd = "${base_sbindir}/rarpd  ${systemd_unitdir}/system/rarpd@.service"
                                     FILES_${PN}-rdisc = "${base_sbindir}/rdisc"
                                     FILES_${PN}-ninfod = "${base_sbindir}/ninfod ${sysconfdir}/init.d/ninfod.sh"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.hidden'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SPLITPKGS = "${PN}-ping ${PN}-arping ${PN}-tracepath \
                                      ${PN}-clockdiff ${PN}-tftpd ${PN}-rdisc \
                                      ${@bb.utils.contains('PACKAGECONFIG', 'rarpd', '${PN}-rarpd', '', d)} \
                                      ${@bb.utils.contains('DISTRO_FEATURES', 'ipv6', '${PN}-traceroute6 ${PN}-ninfod', '', d)}"
                                     PACKAGES += "${SPLITPKGS}"
                                     ALLOW_EMPTY:${PN} = "1"
                                     RDEPENDS_${PN} += "${SPLITPKGS}"
                                     FILES:${PN} = ""
                                     FILES:${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     FILES:${PN}-arping = "${base_bindir}/arping"
                                     FILES:${PN}-tracepath = "${base_bindir}/tracepath"
                                     FILES:${PN}-traceroute6	= "${base_bindir}/traceroute6"
                                     FILES:${PN}-clockdiff = "${base_bindir}/clockdiff"
                                     FILES:${PN}-tftpd = "${bindir} ${base_bindir}/tftpd"
                                     FILES:${PN}-rarpd = "${base_sbindir}/rarpd  ${systemd_unitdir}/system/rarpd@.service"
                                     FILES:${PN}-rdisc = "${base_sbindir}/rdisc"
                                     FILES:${PN}-ninfod = "${base_sbindir}/ninfod ${sysconfdir}/init.d/ninfod.sh"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.hidden'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SPLITPKGS = "${PN}-ping ${PN}-arping ${PN}-tracepath \
                                      ${PN}-clockdiff ${PN}-tftpd ${PN}-rdisc \
                                      ${@bb.utils.contains('PACKAGECONFIG', 'rarpd', '${PN}-rarpd', '', d)} \
                                      ${@bb.utils.contains('DISTRO_FEATURES', 'ipv6', '${PN}-traceroute6 ${PN}-ninfod', '', d)}"
                                     PACKAGES += "${SPLITPKGS}"
                                     ALLOW_EMPTY_${PN} = "1"
                                     RDEPENDS_${PN} += "${SPLITPKGS}"
                                     FILES_${PN} = ""
                                     FILES_${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     FILES_${PN}-arping = "${base_bindir}/arping"
                                     FILES_${PN}-tracepath = "${base_bindir}/tracepath"
                                     FILES_${PN}-traceroute6	= "${base_bindir}/traceroute6"
                                     FILES_${PN}-clockdiff = "${base_bindir}/clockdiff"
                                     FILES_${PN}-tftpd = "${bindir} ${base_bindir}/tftpd"
                                     FILES_${PN}-rarpd = "${base_sbindir}/rarpd  ${systemd_unitdir}/system/rarpd@.service"
                                     FILES_${PN}-rdisc = "${base_sbindir}/rdisc"
                                     FILES_${PN}-ninfod = "${base_sbindir}/ninfod ${sysconfdir}/init.d/ninfod.sh"
                                     ''',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass

# flake8: noqa W291 - we want to explicitly test trailing whitespace here
class TestClassOelintVarsValueQuoted(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.valuequoted'])
    @pytest.mark.parametrize('occurrence', [2])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "a
                                     D = a"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.valuequoted'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A += "b"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGECONFIG[foo] = "-DFOO=ON,-DFOO=OFF,"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'EXTRA_OEMAKE = \'CROSS_COMPILE=${TARGET_PREFIX} CC="${TARGET_PREFIX}gcc ${TOOLCHAIN_OPTIONS}" V=1\'',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     EXTRA_OECMAKE += "\\
                                         -DBUILD_TESTS=OFF \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "\\
                                     a \\
                                     b \\    
                                     c \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsInconSpaces(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.inconspaces'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR += " ffjjj"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR:append = "fhhh"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN}-ptest:append:libc-glibc = "\\
                                     locale-base-en-us.iso-8859-1 \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.inconspaces'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR_append = "fhhh"',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.inconspaces'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR += "ffjjj"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS_append := "foo:"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR:append = " fhhh"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS:append := "foo:"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN}-ptest:append:libc-glibc = "\\
                                         locale-base-en-us.iso-8859-1 \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A += "\\
                                         foo \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.inconspaces'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR_append = " fhhh"',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Item
from oelint_parser.cls_stash import Stash
from oelint_parser.rpl_regex import RegexRpl

from oelint_adv.cls_rule import Rule


class NoSpaceTrailingRule(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.spaces.lineend',
                         severity='warning',
                         message='Line shall not end with a space')

    def __getMatches(self, _file: str, stash: Stash) -> List[Tuple[Item, int]]:
        res = []
        items = stash.GetItemsFor(filename=_file)
        for i in items:
            _linecnt = 0
            for line in i.Raw.split('\n'):
                if line.endswith(' '):
                    res.append((i, i.InFileLine + _linecnt))
                _linecnt += 1
        return res

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        for i in self.__getMatches(_file, stash):
            res += self.finding(i[0].Origin, i[1])
        return res

    def fix(self, _file: str, stash: Stash) -> List[str]:
        res = []
        for i in self.__getMatches(_file, stash):
            i[0].RealRaw = RegexRpl.sub(r'\s+\n', '\n', i[0].RealRaw)
            i[0].Raw = RegexRpl.sub(r'\s+\n', '\n', i[0].Raw)
            res.append(_file)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarInheritDevtool(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.var.inheritdevtool.native'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit native',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit def ${@oe.utils.vartrue("X", "native", "", d)}
                                     ''',
                                 },
                             ],
                             )
    def test_bad_inherit_native(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inheritdevtool.nativesdk'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit nativesdk',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit def ${@oe.utils.vartrue("X", "nativesdk", "", d)}
                                     ''',
                                 },
                             ],
                             )
    def test_bad_inherit_nativesdk(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inheritdevtool.native'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer native
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer ${@oe.utils.ifelse(1 == 1, 'native', C}
                                     ''',
                                 },
                             ],
                             )
    def test_good_inherit_native(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inheritdevtool.nativesdk'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer nativesdk
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer ${@oe.utils.ifelse(1 == 1, 'nativesdk', C}
                                     ''',
                                 },
                             ],
                             )
    def test_good_inherit_nativesdk(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inheritdevtool.cross'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer cross
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer ${@oe.utils.ifelse(1 == 1, 'cross', C}
                                     ''',
                                 },
                             ],
                             )
    def test_good_inherit_cross(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsMispell(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.mispell'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'DPENDS += "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILS = "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRR_URI[sha256sum] = "1234"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.mispell.unknown'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A += "foo"',
                                 },
                             ],
                             )
    def test_bad_unknown(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.mispell'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN} = "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGECONFIG[foo] = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGECONFIG_A = "a"
                                     PACKAGECONFIG_B = "c"
                                     do_configure() {
                                         ./configure ${PACKAGECONFIG_A}
                                     }
                                     python do_foo() {
                                         d.getVar("PACKAGECONFIG_B")
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGES += "${PN}-foo"
                                     INITSCRIPT_PARAMS:${PN}-foo = "bar"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS:qemux86 = "something"

                                     SRC_URI = "git://github.com/znc/znc.git;name=znc;branch=master;protocol=https \\
                                                git://github.com/jimloco/Csocket.git;destsuffix=git/third_party/Csocket;name=Csocket;branch=master;protocol=https"
                                     SRCREV_znc = "bf253640d33d03331310778e001fb6f5aba2989e"
                                     SRCREV_Csocket = "e8d9e0bb248c521c2c7fa01e1c6a116d929c41b4"

                                     ''',
                                 },
                                 {
                                     'abc.bb':
                                     '''
                                     DEPENDS:qemux86 = "something"

                                     SRC_URI = "git://github.com/znc/znc.git;name=abc;branch=master;protocol=https \\
                                                git://github.com/jimloco/Csocket.git;destsuffix=git/third_party/Csocket;name=Csocket;branch=master;protocol=https"
                                     SRCREV_abc = "bf253640d33d03331310778e001fb6f5aba2989e"
                                     SRCREV_Csocket = "e8d9e0bb248c521c2c7fa01e1c6a116d929c41b4"
                                     ''',
                                 },
                                 {
                                     'abc.bb':
                                     '''
                                     SRC_URI = "git://github.com/znc/znc.git;name=abc;branch=master;protocol=https \\
                                                git://github.com/jimloco/Csocket.git;destsuffix=git/third_party/Csocket;name=Csocket;branch=master;protocol=https"
                                     SRCREV_Csocket[doc] = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_install[prefuncs] += "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     dskfdsfkjfhsjkfsdjfkj = "1"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.mispell'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS_${PN} = "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGES += "${PN}-foo"
                                     INITSCRIPT_PARAMS_${PN}-foo = "bar"
                                     ''',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.mispell.unknown'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "a"
                                     B = "c"
                                     do_configure() {
                                         ./configure ${A}
                                     }
                                     python do_foo() {
                                         d.getVar("B")
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_good_unknown(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsPKGSpecific(TestBaseClass):

    def __generate_sample_code(self, var):
        return '''
            {var} = "foo"
            '''.format(var=var)

    @pytest.mark.parametrize('id_', ['oelint.vars.pkgspecific'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'RDEPENDS',
        'RRECOMMENDS',
        'RSUGGESTS',
        'RCONFLICTS',
        'RPROVIDES',
        'RREPLACES',
        'FILES',
        'pkg_preinst',
        'pkg_postinst',
        'pkg_prerm',
        'pkg_postrm',
        'ALLOW_EMPTY',
    ])
    def test_bad(self, id_, occurrence, var):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(var),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pkgspecific'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'RDEPENDS',
        'RRECOMMENDS',
        'RSUGGESTS',
        'RCONFLICTS',
        'RPROVIDES',
        'RREPLACES',
        'FILES',
        'pkg_preinst',
        'pkg_postinst',
        'pkg_prerm',
        'pkg_postrm',
        'ALLOW_EMPTY',
    ])
    def test_bad_append_with_bb(self, id_, occurrence, var):
        input_ = {
            'oelint-adv-test_1.0.bb': 'VAR = "FOO"',
            'oelint-adv-test_1.0.bbappend': self.__generate_sample_code(var),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pkgspecific'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'RDEPENDS_${PN}',
        'RRECOMMENDS_${PN}',
        'RSUGGESTS_${PN}',
        'RCONFLICTS_${PN}',
        'RPROVIDES_${PN}',
        'RREPLACES_${PN}',
        'FILES_${PN}',
        'pkg_preinst_${PN}',
        'pkg_postinst_${PN}',
        'pkg_prerm_${PN}',
        'pkg_postrm_${PN}',
        'ALLOW_EMPTY_${PN}',
    ])
    def test_good(self, id_, occurrence, var):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(var),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pkgspecific'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'RDEPENDS_foo',
        'RRECOMMENDS_foo',
        'RSUGGESTS_foo',
        'RCONFLICTS_foo',
        'RPROVIDES_foo',
        'RREPLACES_foo',
        'FILES_foo',
        'pkg_preinst_foo',
        'pkg_postinst_foo',
        'pkg_prerm_foo',
        'pkg_postrm_foo',
        'ALLOW_EMPTY_foo',
    ])
    def test_good_bbappend(self, id_, occurrence, var):
        input_ = {
            'oelint_adv_test_%.bbappend': self.__generate_sample_code(var),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', [
        'oelint.var.pkgspecific.RDEPENDS',
        'oelint.var.pkgspecific.RRECOMMENDS',
        'oelint.var.pkgspecific.RSUGGESTS',
        'oelint.var.pkgspecific.RCONFLICTS',
        'oelint.var.pkgspecific.RPROVIDES',
        'oelint.var.pkgspecific.RREPLACES',
        'oelint.var.pkgspecific.FILES',
        'oelint.var.pkgspecific.pkg_preinst',
        'oelint.var.pkgspecific.pkg_postinst',
        'oelint.var.pkgspecific.pkg_prerm',
        'oelint.var.pkgspecific.pkg_postrm',
        'oelint.var.pkgspecific.ALLOW_EMPTY',
    ])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGES += "abc-foo"
                                     RDEPENDS_abc-foo += "bar"
                                     ''',
                                 },
                             ],
                             )
    def test_good_custom_pkg(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import argparse
from typing import List

from oelint_adv.data import known_variable_mod, layer_var_mods


class Tweaks:
    """Release specific tweaks"""

    DEFAULT_RELEASE = 'styhead'
    DEVELOPMENT_RELEASE = 'walnascar'

    _map = {
        "inky": {},
        "clyde": {},
        "blinky": {},
        "pinky": {},
        "purple": {},
        "green": {},
        "laverne": {},
        "bernard": {},
        "edison": {},
        "denzil": {},
        "danny": {},
        "dylan": {},
        "dora": {},
        "daisy": {},
        "dizzy": {},
        "fido": {},
        "jethro": {},
        "krogoth": {},
        "morty": {},
        "pyro": {},
        "rocko": {},
        "sumo": {},
        "thud": {},
        "warrior": {},
        "zeus": {},
        "dunfell": {},
        "gatesgarth": {},
        "hardknott": {},
        "honister": {},
        "kirkstone": {"_stash_args": {"new_style_override_syntax": True}},
        "langdale": {},
        "mickledore": {},
        "nanbield": {"constantmods": {"-": {"variables": {"suggested": ["AUTHOR"]}}}},
        "scarthgap": {},
        "styhead": {},
        "walnascar": {},
    }

    @staticmethod
    def releases() -> List[str]:
        return sorted(list(Tweaks._map.keys()) + ['latest'])

    @staticmethod
    def tweak_args(args: argparse.Namespace) -> argparse.Namespace:
        """Tweak the passed arguments for specific releases

        Args:
            args (argparse.Namespace): arguments from main

        Returns:
            argparse.Namespace: tweaked arguments
        """
        def recursive_merge(obj_a: dict, obj_b: dict) -> dict:
            if isinstance(obj_b, (list, str, bool)):
                return obj_b
            for k, v in obj_b.items():
                if obj_a.get(k, None) is None:  # pragma: no cover
                    obj_a[k] = {}
                if isinstance(obj_a[k], list):
                    obj_a[k] += recursive_merge(obj_a[k], v)  # pragma: no cover
                else:
                    obj_a[k] = recursive_merge(obj_a[k], v)
            return obj_a

        _tweaked_options = {}
        _release_range = []

        # override the latest alias
        if args.release == 'latest':
            args.release = Tweaks.DEVELOPMENT_RELEASE

        for k, v in Tweaks._map.items():   # pragma: no cover
            _tweaked_options = recursive_merge(_tweaked_options, v)
            _release_range.append(k)
            if k == args.release:
                break

        for k, v in _tweaked_options.items():
            item = getattr(args, k, None)
            if item is not None:
                if isinstance(item, list):   # pragma: no cover
                    item.append(v)
            else:
                setattr(args, k, v)

        # release known var constantmod
        release_mod = known_variable_mod(args.release, 'core')
        if release_mod:
            args.constantmods.insert(0, release_mod)

        mods, third_party = layer_var_mods(args.files, args.release, args.extra_layer)
        for layer in third_party:
            mod = known_variable_mod(args.release, layer)
            if mod:
                args.constantmods.insert(1, f'+{mod}')
        for mod in mods:
            args.constantmods.insert(1, mod)

        setattr(args, '_release_range', _release_range)  # noqa: B010
        args.state.additional_stash_args = getattr(args, '_stash_args', {})
        return args



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsPNBPNUsage(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.pnbpnusage'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://${PN}.patch"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://${PN}.com/${PN}.git"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "https://foo.org/${PN}"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pnbpnusage'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://${PN}.patch"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://${PN}.com/${PN}.git"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "https://foo.org/${PN}"',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.pnbpnusage'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://${BPN}.patch"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://${BPN}.com/${BPN}.git"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "https://foo.org/${BPN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'S = "${WORDKIR}/${BPN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://foo.org/baz.git;name=${PN}-super"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://foo.org/${BPN}.git;name=${PN}-ultra"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)
